import subprocess
import json
import memobird_agent

APP_CONFIG_PATH = "/etc/memobird_agent/machine_monitor.json"
GENERAL_CONFIG_PATH = "/etc/memobird_agent/general.json"


def memo_print(machine):
    config = {}
    try:
        config_f = open(GENERAL_CONFIG_PATH)
        config = json.load(config_f)
        config_f.close()
    except:
        print("Error occurred when parsing general config file, exiting")
        exit(-1)
    if "smart_guid" not in config or "user_id" not in config or "machine_name" not in config:
        print("Machine config file missing item, exiting")
        exit(-1)
    doc = memobird_agent.Document()
    text = config["machine_name"] + " : " + machine + " is unreachable."
    doc.add_text(text)
    doc.print(config["smart_guid"], config["user_id"])


def ping(address):
    response = subprocess.run(["ping", "-c 1", address], stdout=subprocess.PIPE)
    return response.returncode


def main():
    # Open and Parse config
    machine_list = []
    try:
        config_f = open(APP_CONFIG_PATH)
        config = json.load(config_f)
        config_f.close()
        machine_list = config["machines"]
    except:
        print("Error occurred when parsing config file, exiting")
        exit(-1)

    # Scan through configured processes
    for machine in machine_list:
        if ping(machine) != 0:
            memo_print(machine)


if __name__ == "__main__":
    main()



from Crypto.Cipher import DES
from Crypto.Util import Padding
from datetime import datetime
import hashlib
import base64


class Encryption:
    __primary_key = '4398f676'

    @staticmethod
    def decrypt_message(message, date):
        byte_key = Encryption.get_key(date).encode()
        w = DES.new(byte_key, DES.MODE_CBC, byte_key)

        message = base64.b64decode(message.replace("-", "+").encode())
        message = w.decrypt(message)
        message = Padding.unpad(message, 8, 'pkcs7').decode()
        return message

    @staticmethod
    def encrypt_message(message, date):
        byte_key = Encryption.get_key(date).encode()
        w = DES.new(byte_key, DES.MODE_CBC, byte_key)

        message = Padding.pad(message.encode(), 8, 'pkcs7')
        message = w.encrypt(message)
        message = base64.b64encode(message).decode().replace("+", "-")
        return message

    @staticmethod
    def get_key(date):
        try:
            time = datetime.strptime(date, '%Y-%m-%d %H:%M:%S')
        except ValueError:
            time = datetime.strptime(date, '%Y/%m/%d %H:%M:%S')
        date = time.strftime('%Y%m%d%H%M%S')
        key = '4' + date[7:10] + '3' + date[8:11] + '9' + date[9:12] + '8' + date[10:13]+'f676'
        m = hashlib.md5()
        m.update(key.encode())
        key = m.hexdigest()[0:8]
        return key



class CONFIG:
    VERSION = "2.0.5"
    PRINT_URL = 'http://im.memobird.cn/ashx/DesClientInterface.ashx/'
    APP_URL = 'http://im.memobird.cn/wse/wsesmart.asmx'
    SETTING_URL = 'http://im.memobird.cn/ashx/DesClientInterface.ashx/'
    USER_AGENT = '%E5%92%95%E5%92%95%E6%9C%BA/5 CFNetwork/897.15 Darwin/17.5.0'
    XML_TEMPLATE = '<?xml version="1.0" encoding="utf-8"?>\n<soap:Envelope xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance" xmlns:xsd="http://www.w3.org/2001/XMLSchema" xmlns:soap="http://schemas.xmlsoap.org/soap/envelope/">\n<soap:Header><Mysoapheader xmlns="http://tempuri.org/"><UserID>adming</UserID><PassWord>admin20151010Memobird</PassWord></Mysoapheader></soap:Header>\n<soap:Body><OPERATION xmlns="http://tempuri.org/"><msg>MESSAGE</msg></OPERATION></soap:Body></soap:Envelope>\n'
    LOGIN_MESSAGE_TEMPLATE = '{"type": "0","strUserPwd": "PASSWORD","remark": "iPhone","userCode": "USERNAME","ip": "192.168.1.1","version": "1.4.4"}'
    BIND_MACHINE_MESSAGE_TEMPLATE = '{"strGuid":"GUID","intUserID":"USERID"}'
    SOAP_HEADER = '{"SOAPAction": "http://tempuri.org/OPERATION", "User-Agent": "' + USER_AGENT + '", "Content-Type": "text/xml; charset=utf-8", "Accept-Language": "en-us"}'
    XML_RESPONSE_LOCATOR = '{http://schemas.xmlsoap.org/soap/envelope/}Body/{http://tempuri.org/}OPERATIONResponse/{http://tempuri.org/}OPERATIONResult'



from setuptools import setup
import memobird_agent


with open("README.md", "r", encoding='utf-8') as fh:
    long_description = fh.read()

setup(
    name='memobird_agent',
    version=memobird_agent.__version__,
    description='Print document on memobird from python using official API',
    long_description=long_description,
    long_description_content_type="text/markdown",
    url='http://github.com/tcai793/memobird_agent',
    author='Tong Cai',
    author_email='tong.cai.793@outlook.com',
    license='GPLv3',
    packages=['memobird_agent', 'memobird_agent.script'],
    install_requires=[
        'pillow',
        'requests',
        'pycryptodome',
        'pillow'
    ],
    entry_points={
        'console_scripts': [
            'memo_service_monitor = memobird_agent.script.service_monitor:main',
            'memo_machine_monitor = memobird_agent.script.machine_monitor:main'
        ]
    },
    # Classifiers help users find your project by categorizing it.
    #
    # For a list of valid classifiers, see https://pypi.org/classifiers/
    classifiers=[  # Optional
        # How mature is this project? Common values are
        #   3 - Alpha
        #   4 - Beta
        #   5 - Production/Stable
        'Development Status :: 4 - Beta',

        # Indicate who your project is intended for
        'Intended Audience :: Developers',
        'Topic :: Printing',

        # Pick your license as you wish
        'License :: OSI Approved :: GNU General Public License v3 (GPLv3)',

        # Specify the Python versions you support here. In particular, ensure
        # that you indicate whether you support Python 2, Python 3 or both.
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    keywords='print memobird printer',
    zip_safe=False)



import base64
from PIL import Image
import io
import time
from .security import Encryption
import requests
from .config import CONFIG
import json
import xml.etree.ElementTree as ET

__version__ = CONFIG.VERSION
__all__ = ['Document', 'Util']


class Document:
    def __init__(self):
        self.priority = 0
        self.textList = []

    def add_text(self, text="\n", bold=0, font_size=1, underline=0):
        text = str(text)
        bold = int(bold)
        font_size = int(font_size)
        underline = int(underline)
        if text.endswith("\n") is False:
            text += "\n"
        text = text.encode("gbk", 'replace')
        text = base64.b64encode(text).decode()
        self.textList.append(Document._new_print_item(text, bold, font_size, 0, 1, underline))

    def add_qrcode(self, text="\n"):
        text = str(text)
        if text.endswith("\n") is False:
            text += '\n'
        text = text.encode()
        text = base64.b64encode(text).decode()
        self.textList.append(Document._new_print_item(text, print_type=3))

    def add_sticker(self, sticker_id=0):
        sticker_id = int(sticker_id)
        self.textList.append(Document._new_print_item(icon_id=sticker_id, print_type=4))

    def add_picture(self, path):
        image = Image.open(path)
        image = image.transpose(Image.FLIP_TOP_BOTTOM)
        if image.width > 384:
            image = image.resize((384, image.height * 384 // image.width), Image.LANCZOS)
        image = image.convert('1')
        image_byte_array = io.BytesIO()
        image.save(image_byte_array, "BMP")
        image_byte_array = base64.b64encode(image_byte_array.getvalue()).decode()
        self.textList.append(Document._new_print_item(image_byte_array, print_type=5))

    def set_important(self):
        self.priority = 1

    @staticmethod
    def _new_print_item(base_text="IA0K", bold=0, font_size=1, icon_id=0, print_type=1, underline=0):
        return {
            "basetext": base_text,
            "bold": bold,
            "fontSize": font_size,
            "iconID": icon_id,
            "printType": print_type,
            "underline": underline
        }

    def _generate_printing_content(self, smart_guid, user_id, to_user_id=0):
        to_user_id = user_id if to_user_id == 0 else to_user_id
        date = time.strftime("%Y-%m-%d %H:%M:%S")
        smart_guid = Encryption.encrypt_message(str(smart_guid), date)
        user_id = Encryption.encrypt_message(str(user_id), date)
        to_user_id = Encryption.encrypt_message(str(to_user_id), date)
        content = {
            "type": "1",
            "sysDate": date,
            "parameter": {
                "content": {
                    "strDate": date,
                    "content": {
                        "textList": self.textList
                    },
                    "result": 0,
                    "packageCount": 1,
                    "smartGuid": smart_guid,
                    "userId": user_id,
                    "msgType": 1,
                    "packageNo": 1,
                    "command": 3,
                    "toUserId": to_user_id,
                    "priority": 0
                },
                "datatype": "sendpaper"
            }
        }
        return {"msg": json.dumps(content)}

    # returns (code, print_content_id). code is 0 if syntax error occur; 1 if stored in db; 2 if send to machine
    def print(self, smart_guid, user_id, to_user_id=0):
        user_id = int(user_id)
        to_user_id = int(to_user_id)
        print_content = self._generate_printing_content(smart_guid, user_id, to_user_id)
        response = Connection.send_print_command(print_content)
        return response


class Util:
    @staticmethod
    # returns userID if success, else return -1
    def get_user_id(username, password):
        username = str(username)
        password = str(password)
        login_parameter = CONFIG.LOGIN_MESSAGE_TEMPLATE.replace("USERNAME", username).replace("PASSWORD", password)
        result = Connection.send_util_command(login_parameter, "Login")
        if result['code'] is not 1:
            return -1
        return result['user']['userId']

    @staticmethod
    # returns 1 if success, else return -1
    def bind_machine(smart_guid, user_id):
        smart_guid = str(smart_guid)
        user_id = str(user_id)
        parameter = CONFIG.BIND_MACHINE_MESSAGE_TEMPLATE.replace("GUID", smart_guid).replace("USERID", user_id)
        result = Connection.send_util_command(parameter, "ActivationXG")
        if result['code'] is not 1:
            return -1
        return 1

    @staticmethod
    def set_led_buzz_status(buzz, led):
        obj = {
            "buz": "0",
            "smartguid": "",
            "led": "1",
            "speed": "15",
            "datatype": "setbuzled"
        }
        pass


class Connection:
    @staticmethod
    # returns (code, print_content_id). code is 0 if syntax error occur; 1 if stored in db; 2 if send to machine
    def send_print_command(printing_content):
        ret_code = 0
        content_id = None
        response = requests.post(CONFIG.PRINT_URL, printing_content)
        encoded_json = response.content.decode()
        if encoded_json == '参数错误':
            return ret_code, content_id
        json_result = json.loads(encoded_json)
        decrypted_result = json.loads(Encryption.decrypt_message(json_result['data'], json_result['sysDate']))
        return decrypted_result['code'], decrypted_result['PrintContentID']

    @staticmethod
    def send_util_command(parameter, operation):
        date = time.strftime("%Y-%m-%d %H:%M:%S")
        encrypted_parameter = Encryption.encrypt_message(parameter, date)
        message = json.dumps({'sysDate': date, 'parameter': encrypted_parameter})
        send_xml = CONFIG.XML_TEMPLATE.replace("MESSAGE", message).replace("OPERATION", operation)
        response = requests.post(CONFIG.APP_URL, send_xml,
                                 headers=json.loads(CONFIG.SOAP_HEADER.replace("OPERATION", operation)))
        result_xml = ET.fromstring(response.content.decode())

        xml_locator = CONFIG.XML_RESPONSE_LOCATOR.replace("OPERATION", operation)
        encoded_json = json.loads(result_xml.find(xml_locator).text)
        json_result = json.loads(Encryption.decrypt_message(encoded_json['data'], encoded_json['sysDate']))
        return json_result



import subprocess
import json
import memobird_agent
import re

APP_CONFIG_PATH = "/etc/memobird_agent/service_monitor.json"
STATE_FILE_PATH = "/etc/memobird_agent/data/service_monitor.json"
GENERAL_CONFIG_PATH = "/etc/memobird_agent/general.json"


def memo_print(process, state):
    config = {}
    try:
        config_f = open(GENERAL_CONFIG_PATH)
        config = json.load(config_f)
        config_f.close()
    except:
        print("Error occurred when parsing general config file, exiting")
        exit(-1)
    if "smart_guid" not in config or "user_id" not in config or "machine_name" not in config:
        print("Machine config file missing item, exiting")
        exit(-1)
    doc = memobird_agent.Document()
    text = config["machine_name"] + ": " + process + " is " + state
    doc.add_text(text)
    doc.print(config["smart_guid"], config["user_id"])


def main():
    # Open and Parse config
    process_list = []
    try:
        config_f = open(APP_CONFIG_PATH)
        config = json.load(config_f)
        config_f.close()
        process_list = config["processes"]
    except:
        print("Error occurred when parsing config file, exiting")
        exit(-1)

    # Open and Parse data
    state = {}
    try:
        state_f = open(STATE_FILE_PATH, "r")
        state = json.load(state_f)
        state_f.close()
    except:
        print("Cannot read state file")

    # Scan through configured processes ps -e -o command
    running_processes = subprocess.run(["ps", "-e", "-o command"], stdout=subprocess.PIPE).stdout.decode()
    for process in process_list:
        if process in running_processes:
            new_status = "UP"
        else:
            new_status = "DOWN"
        # For each process, if exists in data, then Compare with stored data.
        # If same then continue, else change current state and print
        if process in state:
            if state[process] == new_status:
                continue
            else:
                state[process] = new_status
                memo_print(process, new_status)
        # if DNE in data, then add current state and print
        else:
            state[process] = new_status
            memo_print(process, new_status)

    # Write data back to data location
    try:
        state_f = open(STATE_FILE_PATH, "w")
        json.dump(state, state_f)
    except:
        print("Error occurred when writing process data back to file")
        exit(-1)


if __name__ == "__main__":
    main()
